// Determines the orientation of each cube, given a valid HC in the converted instace. Note that there is never any
// need to reorder the cubes, so the orientations are given in the initial cube order.
//
 // The output shows the four long faces. The cubes can then be orientated to match the four faces. For example:
 // 
 // Face 1 is: 2 1 3 4 6 5
   // Face 2 is: 3 6 4 2 5 1
   // Face 3 is: 4 5 2 1 3 6
   // Face 4 is: 5 3 1 6 4 2
 //
 // The above implies that the first cube should be orientated to have colour 2 on the top, colour 3 on the front, 
 // colour 4 on the bottom and colour 5 on the back. Orienting the six cubes as described above gives the solution.
//
// The input should be a .hcp.tou file, such as the following:
//
// NAME : filename.hcp.tou
// COMMENT : HC for filename.hcp
// TYPE : TOUR
// DIMENSION : 1200
// TOUR_SECTION
// 1
// 2
// 3
// ...
// 1200
// -1
// EOF
//
//
// Written by Michael Haythorpe

import java.io.*;
import java.util.StringTokenizer;

public class II2HCPSol
{
   public static void main(String [] args)
   {
      if(args.length != 1)
      {
         System.out.println("ERROR: Input must be of the following form: II2HCPSol <output.hcp.tou>");
         return;
      }
   
      String outputFilename = args[0];
      BufferedReader br;
   
      try
      {
         br = new BufferedReader(new FileReader(outputFilename));
      }
      catch(Exception e)
      {
         System.out.println("ERROR: Cannot open file " + outputFilename);
         return;
      }
         
      String line = "";
   		
      try
      {
         line = br.readLine();
         line = br.readLine();
         line = br.readLine();
         line = br.readLine();
         StringTokenizer tokens = new StringTokenizer(line);
         tokens.nextToken();
         tokens.nextToken();
         int nodes = Integer.parseInt(tokens.nextToken());
         int N = ((nodes/3)-4)/66;
         if(3*(N*66+4) != nodes)
         {
            System.out.println("ERROR: Incorrect number of nodes!");
            return;
         }
         line = br.readLine();
         
         int [][]arcs = new int[nodes][2];
         int []degrees = new int[nodes];
      	
         line = br.readLine();
         int previous = Integer.parseInt(line)-1;
         line = br.readLine();
      
         while(!line.equals("-1"))
         {
            int current = Integer.parseInt(line)-1;
            arcs[current][degrees[current]++] = previous;
            arcs[previous][degrees[previous]++] = current;
            line = br.readLine();
            previous = current;
         }
         
         arcs[0][1] = previous;
         arcs[previous][1] = 0;
         br.close();
         
      	// Now to pair them all up
      	
         int [][]faces = new int[4][N];
      	
         for(int i=0; i<N; i++)
         {
            int node = 3+6*i+3;
            int node2 = 3+6*i+4;
         
            if(arcs[node-1][0] == node-2)
            {
               faces[0][(arcs[node-1][1] - (12*N + 10))/84] = i+1;
            }
            else
            {
               faces[0][(arcs[node-1][0] - (12*N + 10))/84] = i+1;
            }
            
            if(arcs[node2-1][0] == node2)
            {
               faces[2][(arcs[node2-1][1] - (12*N + 10))/84] = i+1;
            }
            else
            {
               faces[2][(arcs[node2-1][0] - (12*N + 10))/84] = i+1;
            }
         
         }
         
         for(int i=0; i<N; i++)
         {
            int node = 6*N + 3 + 3+6*i+3;
            int node2 = 6*N + 3 + 3+6*i+4;
         
            if(arcs[node-1][0] == node-2)
            {
               faces[1][(arcs[node-1][1] - (96*N + 10))/102] = i+1;
            }
            else
            {
               faces[1][(arcs[node-1][0] - (96*N + 10))/102] = i+1;
            }
            
            if(arcs[node2-1][0] == node2)
            {
               faces[3][(arcs[node2-1][1] - (96*N + 10))/102] = i+1;
            }
            else
            {
               faces[3][(arcs[node2-1][0] - (96*N + 10))/102] = i+1;
            }
         
         }
      	
         for(int i=0; i<4; i++)
         {
            System.out.print("Face " + (i+1) + " is: ");
            for(int j=0; j<N; j++)
            {
               for(int k=(int)Math.floor(Math.log10(faces[i][j])); k<(int)Math.floor(Math.log10(N)); k++)
                  System.out.print(" ");
               System.out.print(faces[i][j] + " ");
            
            }
            System.out.println();  
         }
      	
      	
      	
      }
      catch(Exception e)
      {
         System.out.println("ERROR: Tour file is not in proper format: " + line);
         return;
      }
   
   }
}